var fs = require('fs');
var http = require('http');
var https = require('https');
var vows = require('vows');
var assert = require('assert');
var speck = require('speck');
var mountableProxy = require('../lib/proxy');

var conf = {
  http: {
    host: 'localhost',
    port: 8081
  },
  https: {
    host: 'localhost',
    port: 443,
    key: fs.readFileSync('./spec/fixtures/proxy.key', 'utf8'),
    cert: fs.readFileSync('./spec/fixtures/proxy.cert', 'utf8'),
  }
};

var config = {
  host: 'localhost',
  port: 8082,
  root: '/_'
};

vows
  .describe('PROXY')
  .addBatch({
    'A Proxy': {
      topic: function() {
        var client = new speck.Klient({
          host: conf.http.host,
          port: conf.http.port,
          server: http.createServer(function(req, res) {
            mountableProxy.createProxy(config).http(req, res);
          })
        });

        return client;
      },
      'connected against a foreign server': {
        topic: function(client, proxy) {
          var server = http.createServer();

          return server;
        },
        'which is not running': {
          topic: function(server, client) {
            return client.get({ path: '/' });
          },
          'should return *500*':
            speck.makros.status(500),
          'should respond with *application/json*':
            speck.makros.header('content-type', 'application/json'),
          'should return *error*': function(err, res) {
            assert.ifError(err);
            if (/^v0\.4/.test(process.version)) {
              assert.deepEqual(res.body, { error: 'ECONNREFUSED, Connection refused' });
            } else {
              assert.deepEqual(res.body, { error: 'connect ECONNREFUSED' });
            }
          }
        }
      },
      teardown: function(client) {
        client.close();
      }
    }
  })
  .addBatch({
    'A Proxy': {
      topic: function() {
        var client = new speck.Klient({
          host: conf.http.host,
          port: conf.http.port,
          server: http.createServer(function(req, res) {
            mountableProxy.createProxy(config).http(req, res);
          })
        });

        return client;
      },
      'connected against a foreign server': {
        topic: function(client, proxy) {
          var server = http.createServer();

          return server;
        },
        'which is running': {
          topic: function(server, client, proxy) {
            server.listen(config.port, config.host, this.callback);
          },
          'and a request': {
            topic: function(server, client, proxy) {
              var that = this;
              var reqBody = JSON.stringify({ foo: 'bar' });

              server.once('request', function(req, res) {
                var body = '';

                req.on('data', function(data) {
                  body += data;
                });

                req.on('end', function() {
                  if (req.headers['content-type'] === 'application/json') {
                    body = JSON.parse(body);
                  }

                  req.body = body;

                  res.writeHead(200, {
                    'content-length': 'ok'.length,
                    'content-type': 'text/plain'
                  });
                  res.end('ok');

                  that.callback(null, req);
                });
              });

              client.post({
                path: '/_/foo',
                headers: {
                  'content-length': reqBody.length,
                  'content-type': 'application/json'
                },
                data: reqBody
              });
            },
            'should change *Host*':
              speck.makros.header('host', config.host),
            'should pass *body*':
              speck.makros.body({ foo: 'bar' }),
            'should rewrite *URL*': function (err, req) {
              assert.ifError(err);
              assert.equal(req.url, '/foo');
            }
          },
          teardown: function(server, client) {
            server.close();
            client.close();
          }
        }
      }
    }
  })
  .addBatch({
    'A Proxy': {
      topic: function() {
        var client = new speck.Klient({
          host: conf.http.host,
          port: conf.http.port,
          server: http.createServer(function(req, res) {
            mountableProxy.createProxy(config).http(req, res);
          })
        });

        return client;
      },
      'connected against a foreign server': {
        topic: function(client, proxy) {
          var server = http.createServer();

          return server;
        },
        'which is running': {
          topic: function(server, client, proxy) {
            server.listen(config.port, config.host, this.callback);
          },
          'and response': {
            topic: function(server, client) {
              var resBody = JSON.stringify({ foo: 'bar', bar: 'baz' });

              server.once('request', function(req, res) {
                req.on('end', function() {
                  res.writeHead(201, {
                    'content-length': resBody.length,
                    'content-type': 'application/json',
                    'x-foo': 'bar'
                  });

                  res.end(resBody);
                });
              });

              return client.get({ path: '/' });
            },
            'should pass *201*':
              speck.makros.status(201),
            'should pass *header*':
              speck.makros.header('x-foo', 'bar'),
            'should pass *body*':
              speck.makros.body({ foo: 'bar', bar: 'baz' })
          }
        },
        teardown: function (server, client) {
          server.close();
          client.close();
        }
      }
    }
  })
  .addBatch({
    'A Proxy': {
      topic: function() {
        var client = new speck.Klient({
          host: conf.http.host,
          port: conf.http.port,
          server: http.createServer(function(req, res) {
            mountableProxy.createProxy(config).http(req, res);
          })
        });

        return client;
      },
      'connected against a foreign server': {
        topic: function(client, proxy) {
          var server = http.createServer();

          return server;
        },
        'which is running': {
          topic: function(server, client, proxy) {
            server.listen(config.port, config.host, this.callback);
          },
          'and response from redirect': {
            topic: function(server, client) {
              server.once('request', function(req, res) {
                req.on('end', function() {
                  res.writeHead(301, {
                    'location': 'http://' + config.host + '/foobarbaz'
                  });

                  res.end();
                });
              });

              return client.get({ path: '/_/foobar' });
            },
            'should respond with *301*':
              speck.makros.status(301),
            'should change *location*':
              speck.makros.header('location', '/_/foobarbaz')
          }
        },
        teardown: function (server, client) {
          server.close();
          client.close();
        }
      }
    }
  })
  .addBatch({
    'A Proxy': {
      topic: function() {
        var that = this;
        var server;

        server = https.createServer(conf.https);

        server.on('request', function(req, res) {
          mountableProxy.createProxy(config).https(req, res, { port: config.port });
        });

        server.listen(conf.http.port, conf.http.host, that.callback);
      },
      'connected against a foreign server': {
        topic: function(client, proxy) {
          var server;

          server = https.createServer(conf.https);

          return server;
        },
        'which is running': {
          topic: function(server) {
            server.listen(config.port, config.host, this.callback);
          },
          'and respond with a *304*': {
            topic: function(server) {
              var that = this;
              var req;

              server.on('request', function(req, res) {
                req.on('end', function() {
                  res.writeHead(304, {
                    "server":"nginx",
                    "date":"Wed, 17 Aug 2011 13:18:50 GMT",
                    "connection":"close",
                    "x-runtime":"23",
                    "etag":"\"1958d85c0fa835b4710b45338ac9fc29\"",
                    "cache-control":"private, max-age=0, must-revalidate",
                    "access-control-allow-origin":"*",
                    "access-control-allow-methods":"GET, PUT, POST, DELETE",
                    "access-control-allow-headers":"Authorization, Content-Type, Accept"
                  });

                  res.end();
                });
              });

              req = https.request({
                host: conf.http.host,
                port: conf.http.port,
                path: '/_/foobar',
                method: 'GET'
              });

              req.on('error', function(error) {
                that.callback(error);
              });

              req.on('response', function(response) {
                var body = '';
                var cb = function() {
                  response.body = body;

                  that.callback(null, response);
                };

                response.on('data', function(data) {
                  body += data.toString('utf8');
                });
                response.on('close', cb);
                response.on('end', cb);
              });

              req.end();
            },
            'should respond': function(error, response) {
              assert.ifError(error);
              assert.equal(304, response.statusCode);
            }
        }
        },
        teardown: function (server) {
          server.close();
        }
      }
    }
  })
  .export(module);

